/* Stuff specific to the Interlan NI5010 Ethernet controller board */
/* B. Doster - First pass 3/9/88 */

#define	IL_MAX	1	/* Only one controller supported */
extern unsigned nil;	/* Actual number of Ethernet controllers */

/* Find base address of specified device */
#define	IE(dev)	(il_dev[dev].addr)

/* The various IE command registers */
#define	EDLC_XSTAT(num)	(num)		/* EDLC transmit csr */
#define	EDLC_XCLR(num)	(num)		/* EDLC transmit "Clear IRQ" */
#define	EDLC_XMASK(num)	((num)+0x1)	/* EDLC transmit "IRQ Masks" */
#define	EDLC_RSTAT(num)	((num)+0x2)	/* EDLC receive csr */
#define	EDLC_RCLR(num)	((num)+0x2)	/* EDLC receive "Clear IRQ" */
#define	EDLC_RMASK(num)	((num)+0x3)	/* EDLC receive "IRQ Masks" */
#define	EDLC_XMODE(num)	((num)+0x4)	/* EDLC transmit Mode */
#define	EDLC_RMODE(num)	((num)+0x5)	/* EDLC receive Mode */
#define	EDLC_RESET(num)	((num)+0x6)	/* EDLC RESET register */
#define	EDLC_TDR1(num)	((num)+0x7)	/* "Time Domain Reflectometry" reg1 */
#define	EDLC_ADDR(num)	((num)+0x8)	/* EDLC station address, 6 bytes */
					/* 0xE doesn't exist for r/w */
#define	EDLC_TDR2(num)	((num)+0xf)	/* "Time Domain Reflectometry" reg2 */
#define	IE_GP(num)	((num)+0x10)	/* GP pointer (word register) */
					/* 0x11 is 2nd byte of GP Pointer */
#define	IE_RCNT(num)	((num)+0x10)	/* Count of bytes in rcv'd packet */
					/* 0x11 is 2nd byte of "Byte Count" */
#define	IE_MMODE(num)	((num)+0x12)	/* Memory Mode register */
#define IE_DMA_RST(num)	((num)+0x13)	/* IE DMA Reset.  write only */
#define	IE_ISTAT(num)	((num)+0x13)	/* IE Interrupt Status.  read only */
#define	IE_RBUF(num)	((num)+0x14)	/* IE Receive Buffer port */
#define	IE_XBUF(num)	((num)+0x15)	/* IE Transmit Buffer port */
#define	IE_SAPROM(num)	((num)+0x16)	/* window on station addr prom */
#define	IE_RESET(num)	((num)+0x17)	/* any write causes Board Reset */

/* bits in EDLC_XSTAT, interrupt clear on write, status when read */
#define XS_TPOK		0x80		/* transmit packet successful */
#define XS_CS		0x40		/* carrier sense */
#define XS_RCVD		0x20		/* transmitted packet received */
#define XS_SHORT	0x10		/* transmission media is shorted */
#define XS_UFLW		0x08		/* underflow.  iff failed board */
#define XS_COLL		0x04		/* collision occurred */
#define XS_16COLL	0x02		/* 16th collision occurred */
#define XS_PERR		0x01		/* parity error */

#define XS_CLR_UFLW	0x08		/* clear underflow */
#define XS_CLR_COLL	0x04		/* clear collision */
#define XS_CLR_16COLL	0x02		/* clear 16th collision */
#define XS_CLR_PERR	0x01		/* clear parity error */

/* bits in EDLC_XMASK, mask/enable transmit interrupts.  register is r/w */
#define	XM_TPOK		0x80		/* =1 to enable Xmt Pkt OK interrupts */
#define	XM_RCVD		0x20		/* =1 to enable Xmt Pkt Rcvd ints */
#define	XM_UFLW		0x08		/* =1 to enable Xmt Underflow ints */
#define	XM_COLL		0x04		/* =1 to enable Xmt Collision ints */
#define	XM_COLL16	0x02		/* =1 to enable Xmt 16th Coll ints */
#define	XM_PERR		0x01		/* =1 to enable Xmt Parity Error ints */
					/* note: always clear this bit */

/* bits in EDLC_RSTAT, interrupt clear on write, status when read */
#define	RS_PKT_OK	0x80		/* received good packet */
#define	RS_RST_PKT	0x10		/* RESET packet received */
#define	RS_RUNT		0x08		/* Runt Pkt rcvd.  Len < 64 Bytes */
#define	RS_ALIGN	0x04		/* Alignment error. not 8 bit aligned */
#define RS_CRC_ERR	0x02		/* Bad CRC on rcvd pkt */
#define	RS_OFLW		0x01		/* overflow for rcv FIFO */
#define	RS_VALID_BITS	(RS_PKT_OK|RS_RST_PKT|RS_RUNT|RS_ALIGN|RS_CRC_ERR|RS_OFLW)
					/* all valid RSTAT bits */

#define RS_CLR_PKT_OK	0x80		/* clear rcvd packet interrupt */
#define	RS_CLR_RST_PKT	0x10		/* clear RESET packet received */
#define	RS_CLR_RUNT	0x08		/* clear Runt Pckt received */
#define	RS_CLR_ALIGN	0x04		/* clear Alignment error */
#define	RS_CLR_CRC_ERR	0x02		/* clear CRC error */
#define	RS_CLR_OFLW	0x01		/* clear rcv FIFO Overflow */

/* bits in EDLC_RMASK, mask/enable receive interrupts.  register is r/w */
#define	RM_PKT_OK	0x80		/* =1 to enable rcvd good packet ints */
#define	RM_RST_PKT	0x10		/* =1 to enable RESET packet ints */
#define	RM_RUNT		0x08		/* =1 to enable Runt Pkt rcvd ints */
#define	RM_ALIGN	0x04		/* =1 to enable Alignment error ints */
#define RM_CRC_ERR	0x02		/* =1 to enable Bad CRC error ints */
#define	RM_OFLW		0x01		/* =1 to enable overflow error ints */

/* bits in EDLC_RMODE, set Receive Packet mode.  register is r/w */
#define	RMD_TEST	0x80		/* =1 for Chip testing.  normally 0 */
#define	RMD_ADD_SIZ	0x10		/* =1 5-byte addr match.  normally 0 */
#define	RMD_EN_RUNT	0x08		/* =1 enable runt rcv.  normally 0 */
#define	RMD_EN_RST	0x04		/* =1 to rcv RESET pkt.  normally 0 */

#define	RMD_PROMISC	0x03		/* receive *all* packets.  unusual */
#define	RMD_MULTICAST	0x02		/* receive multicasts too.  unusual */
#define	RMD_BROADCAST	0x01		/* receive broadcasts & normal. usual */
#define	RMD_NO_PACKETS	0x00		/* don't receive any packets. unusual */

/* bits in EDLC_XMODE, set Transmit Packet mode.  register is r/w */
#define	XMD_COLL_CNT	0xF0		/* coll's since success.  read-only */
#define	XMD_IG_PAR	0x08		/* =1 to ignore parity.  ALWAYS set */
#define	XMD_T_MODE	0x04		/* =1 to power xcvr. ALWAYS set this */
#define	XMD_LBC		0x02		/* =1 for loopbakc.  normally set */
#define	XMD_DIS_C	0x01		/* =1 disables contention. normally 0 */

/* bits in EDLC_RESET, write only */
#define	RS_RESET	0x80		/* =1 to hold EDLC in reset state */

/* bits in IE_MMODE, write only */
#define	MM_EN_DMA	0x80		/* =1 begin DMA xfer, Cplt clrs it */
#define	MM_EN_RCV	0x40		/* =1 allows Pkt rcv.  clr'd by rcv */
#define	MM_EN_XMT	0x20		/* =1 begin Xmt pkt.  Cplt clrs it */
#define	MM_BUS_PAGE	0x18		/* =00 ALWAYS.  Used when MUX=1 */
#define	MM_NET_PAGE	0x06		/* =00 ALWAYS.  Used when MUX=0 */
#define	MM_MUX		0x01		/* =1 means Rcv Buff on system bus */
					/* =0 means Xmt Buff on system bus */

/* bits in IE_ISTAT, read only */
#define	IS_TDIAG	0x80		/* =1 if Diagnostic problem */
#define	IS_EN_RCV	0x20		/* =1 until frame is rcv'd cplt */
#define	IS_EN_XMT	0x10		/* =1 until frame is xmt'd cplt */
#define	IS_EN_DMA	0x08		/* =1 until DMA is cplt or aborted */
#define	IS_DMA_INT	0x04		/* =0 iff DMA done interrupt.  */
#define	IS_R_INT	0x02		/* =0 iff unmasked Rcv interrupt */
#define	IS_X_INT	0x01		/* =0 iff unmasked Xmt interrupt */

/* miscellaneous sizes */
#define	BFRSIZ		2048	/* number of bytes in a buffer */

struct estats {
	long recv;		/* Good packets received */
	long bad;		/* Bad receive packets */
	long over;		/* Overflow errors */
	long drop;		/* Dropped because RX queue too long */
	long nomem;		/* Dropped because buffer malloc failed */
	long intrpt;		/* Interrupts */
	long xmit;		/* Total output packets */
	long timeout;		/* Transmitter timeouts */
	long jam;		/* Collisions */
	long jam16;		/* 16 successive collisions */
	long rstpkt;		/* Reset Packets received */
	long runt;		/* Runt packets received */
	long dmadone;		/* Count of DMA Completes (should be zero) */
};
struct il {
	unsigned base;		/* Base I/O address */
	unsigned vec;		/* Interrupt vector */
	struct mbuf *rcvq;	/* Queue of incoming packets */
	unsigned rcvcnt;	/* Number of packets on queue */
	unsigned rcvmax;	/* Maximum length of rcvq */
	struct estats estats;	/* Controller statistics */
	short size;		/* Size of current transmit packet */
	struct interface *iface;
};
extern struct il il[];
